# 第七章（2）构建WebSocket

* WebSocket服务端可以推送数据到客户端，这远比HTTP请求响应模式更灵活、更高效，

* WebSocket的握手部分是由HTTP完成的，

* 一旦WebSocket握手成功，服务端与客户端将会呈现对等的效果，能接受和发送消息

## WebSocket握手（进行HTTP交互）
### 客户端请求报文
```javascript
GET /chat HTTP/1.1
Host: server.example.com
Upgrade: websocket
Connection: Upgrade
Sec-WebSocket-Key: dGhlIHNhbXBsZSBub25jZQ==
Sec-WebSocket-Protocol: chat, superchat
Sec-WebSocket-Version: 13

实现代码如下：
var WebSocket = function (url) 
{
    // 􀿁代码􀇈解析ws://127.0.0.1:12010/updates􀇈用􁇀请求
    this.options = parseUrl(url);
    this.connect();
 };
WebSocket.prototype.onopen = function () {
    // TODO
    };
WebSocket.prototype.setSocket = function (socket) {
    this.socket = socket;
  };
WebSocket.prototype.connect = function () 
{
        var this = that;
        var key = new Buffer(this.options.protocolVersion + '-' + Date.now()).toString('base64');
 var shasum = crypto.createHash('sha1');

 var expected = shasum.update(key + '258EAFA5-E914-47DA-95CA-C5AB0DC85B11').digest('base64');
  var options = 
    {
        port: this.options.port, // 12010
        host: this.options.hostname, // 127.0.0.1
        headers: 
        {
            'Connection': 'Upgrade',
            'Upgrade': 'websocket',
            'Sec-WebSocket-Version': this.options.protocolVersion,
            'Sec-WebSocket-Key': key
        }
    };
    var req = http.request(options);
    req.end();
    req.on('upgrade', function(res, socket, upgradeHead)
     {
        //连接成功
        that.setSocket(socket);
        //触发open事件
        that.onopen();
        });
};
```

### 服务端响应报文

```javascript
HTTP/1.1 101 Switching Protocols
Upgrade: websocket
Connection: Upgrade
Sec-WebSocket-Accept: s3pPLMBiTxaQ9kYGzzhZRbK+xOo=
Sec-WebSocket-Protocol: chat

实现代码如下：
var server = http.createServer(function (req, res)
{
    res.writeHead(200, {'Content-Type': 'text/plain'});
    res.end('Hello World\n');
});
    server.listen(12010);
// 在收􀚟upgrade请求􀢫􀇈􀟢􁊮客户端􁈎􁂹􀵎换􁁹􁅱
server.on('upgrade', function (req, socket, upgradeHead) 
{
    var head = new Buffer(upgradeHead.length);
    upgradeHead.copy(head);
    var key = req.headers['sec-websocket-key'];
    var shasum = crypto.createHash('sha1');
    key = shasum.update(key + "258EAFA5-E914-47DA-95CA-C5AB0DC85B11").digest('base64');
    var headers =
     [
    'HTTP/1.1 101 Switching Protocols',
    'Upgrade: websocket',
    'Connection: Upgrade',
    'Sec-WebSocket-Accept: ' + key,
    'Sec-WebSocket-Protocol: ' + protocol
    ];
   //数据立即发送
    socket.setNoDelay(true);
    socket.write(headers.concat('', '').join('\r\n'));
    // 建立服务器端的WebSocket连接
    var websocket = new WebSocket();
    websocket.setSocket(socket);
});
```
## WebSocket数据传输

* 握手成功后，当前连接将不再进行HTTP的交互

（1） 握手完成后，客户端的onopen（）将会被触发执行
```javaScript
socket.onopen = function () {
// TODO: opened()
};
```

（2） WebSocket数据协议帧都是在底层data事件上封装完成的

* 接受
```javascript
WebSocket.prototype.setSocket = function (socket) {
this.socket = socket;
this.socket.on('data', this.receiver);
};
```

* 发送
```javascript
WebSocket.prototype.send = function (data) {
this._send(data);
};

客户端发送，服务器端会触发onmessage()。
```

* 客户端发送数据帧需要进行掩码处理，服务端发送到客户端的数据无须做掩码处理。

```
fin(1) + res(000) + opcode(0001) + masked(1) + payload length(1100000) + masking key(32位) + payload
data(hello world!加密􀢫的􀜾进制)
```



