### 在Node中引入模块，需要经历如下3个步骤：

（1） 路径分析

（2） 文件定位：（package.json文件，运用JSON.parse解析出包描述对象，进行文件名定位）

（3） 编译和执行:

* 定位到具体文件后，Node会新建一个模块对象，然后根据路径载入并编译。Node会根据不同的文件扩展名（.js .node .json）调用不同的具体编译方式执行后返回给调用者。Module._extensions来确定对应扩展名文件的加载方式.

  - JavaScript模块编译过程中，Node对获取的Javascript文件内容进行了头尾包装。

    > 编译成功后,模块都会将其文件路径作为索引缓存到NativeModule._cache对象上（文件模块是缓存在Module._cache对象上）

  - C/C++模块的编译时，实际上Node调用process.dlopen（）方法对***现有的库***进行加载即可。

  - Json文件的编译:Node利用fs模块“同步”读取JSON文件的内容之后，调用JSON.parse()方法得到对象，然后将它赋给模块对象的exports,以供外部调用。

> 核心模块在Node源代码编译过程中，已经编译进了二进制文件，所以Node进程启动时，部分核心代码直接就加载进内存中，所以这部分核心代码在引入时，（1）文件定位、（2）编译执行这两个步骤可以省略掉，并且路径分析优先判断，所以加载速度是最快的。

###  一. javascript“核心模块”的编译过程


1.  在编译所有c/c++文件之前，编译程序首先将javascript模块文件编译成C/C++代码（Node采用了V8附带的***js2c.py工具***，将所有内置的Javacript代码——src/node.js；lib/*.js，转换成C++里的数组，生成node_natives.h头文件）。

  ```c++
  namespace node
  {
      const char node_native[] = { 47, 47, ..};
      const char dgram_native[] = { 47, 47, ..};
      const char console_native[] = { 47, 47, ..};
      const char buffer_native[] = { 47, 47, ..};
      const char querystring_native[] = { 47, 47, ..};
      const char punycode_native[] = { 47, 42, ..};
    ...
    struct _native {
    const char* name;
    const char* source;
    size_t source_len;
    };
  static const struct _native natives[] =
  {
      { "node", node_native, sizeof(node_native)-1 },
      { "dgram", dgram_native, sizeof(dgram_native)-1 },
    ...
  };
  }
  ```

2. 然后，应该是被编译成了二进制文件了，但是，是***不可执行代码***。

3. 启动node 进程时，javacript代码经历标识符分析后定位（应该可以理解成加载）到内存中。

4. 为了得到最终编译好的javascript模块，需要：
   - （1）从内存中获取源码——源文件通过process.binding('natives')取出。

    ```javascript
      function NativeModule(id) 
      {
        this.filename = id + '.js';
        this.id = id;
        this.exports = {};
        this.loaded = false;
      }
      NativeModule._source = process.binding('natives');
      NativeModule._cache = {};
      ```

   `NativeModule._source = process.binding('natives');`//该方法将通过***js2c.py工具***转化出的字符串数组取出，然后重新转换成普通字符串，以对Javascript核心模块进行编译和执行。

   - （2）javascript核心模块开始编译，需要经历头尾包装：

   ```javascript
      (function (exports, require, module, __filename, __dirname) {
      var math = require('math');
      exports.area = function (radius) {
      return Math.PI * radius * radius;
      };
      });
   ```

5. 编译成功后，模块都会将其文件路径作为索引缓存到NativeModule._cache对象上。

### 二. C++核心模块的编译过程

1. 由纯C/C++编写的部分统一称之为内建模块，其组织形式如下：

      ```c++
      struct node_module_struct {
      int version;
      void *dso_handle;
      const char *filename;
      void (*register_func) (v8::Handle<v8::Object> target);
      const char *modname;
      };
      ```

2. 每个内建模块定义之后：

  - （1）通过NODE_MODULE宏将模块定义到node命名空间中；
  - （2）该模块方法需要挂载到当前结构为register_func的成员：

    ```c++
    //关于宏NODE_MODULE的定义如下：
    #define NODE_MODULE(modname, regfunc) \
    extern "C" { \
    NODE_MODULE_EXPORT node::node_module_struct modname ## _module = \
    { \
    NODE_STANDARD_MODULE_STUFF, \
    regfunc, \
    NODE_STRINGIFY(modname) \
    }; \
    }
    //我们在编写核心模块时，只需调用NODE_MODULE(modname, regfunc)即可。
    ```

3. 将编写好的内建模块统一的放进node_extensions.h下的***node_module_list***的数组中,其中函数名必须以“node”开头。

    ```c++
    node_buffer
    􀂉 node_crypto
    􀂉 node_evals
    􀂉 node_fs
    􀂉 node_http_parser
    􀂉 node_os
    􀂉 node_zlib
    􀂉 node_timer_wrap
    􀂉 node_tcp_wrap
    􀂉 node_udp_wrap
    ```

4. 进行文件编译，则会编进二进制文件中。

5. 启动Node时，核心代码就会直接加载进内存，供我们调用：

   ```javascript
   注意：
   (1) 核心模块基本都封装了内建模块，外部javascript核心模块一般是调用内建模块；
   (2)Node启动时，会生成全局变量process，并提供Binding()方法，来协助“加载内建模块”；Binding()的实现代码在src/node.cc中。其中：
 
          static Handle<Value> Binding(const Arguments& args)
          {
                  (1) 调用：`get_builtin_module()` 方法取出内建模块对象
                  (2) 执行register_func()填充exports对象
                  (3) 最后将exports对象按模块名缓存，并返回给调用，完成导出。
          }
 
         get_builtin_module()获取内建模块对象、执行register_func()填充exports对象、最后将exports对象按模块名缓存，并返回给调用方完成导出。
   ```

内建模块优势：

（1） 一旦Node开始运行，它们直接加载进内存中。

（2）无须再次错标识符定位、文件定位、编译等过程

（3）直接就可执行。

###  三. 自己编写核心模块与编译

1. 编写头文件,保存为node_hello.h

```c++
  #ifndef NODE_HELLO_H_
  #define NODE_HELLO_H_
  #include <v8.h>
  namespace node {
  // 􁇨定义方法
  v8::Handle<v8::Value> SayHello(const v8::Arguments& args);
  }
#endif
```

2. 编写node_hello.cc,并存储到src目录下：

```javascript
  #include <node.h>
  #include <node_hello.h>
  #include <v8.h>
  namespace node 
  {
    using namespace v8;
  // 实现􁇨定义的方法
    Handle<Value> SayHello(const Arguments& args)
    {
    HandleScope scope;
    return scope.Close(String::New("Hello world!"));
    }
    // 给传入的目􀕔对象􀽁加sayHello方法
    void Init_Hello(Handle<Object> target) 
    {
    target->Set(String::NewSymbol("sayHello"), FunctionTemplate::New(SayHello)->GetFunction());
    }
  }
  // 调用NODE_MODULE()将注􀖩方法定义到内存中
    NODE_MODULE(node_hello, node::Init_Hello)
```
3. 编译

*  更改src/node_extensions.h文件，在NODE_EXT_LIST_END前添加NODE_EXT_LIST_ITEM(node_hello),从而将node_hello模块添加进node_module_list数组中。

* 更改Node的项目生成文件node.gyp，并在'target_name': 'node'点节的sources中添加***新编写的两个文件***，然后编译整个Node项目。

4. 运行

```javascript
    `$ node
    > var hello = process.binding('hello');
    undefined
    > hello.sayHello();
    'Hello world!'
    >`
```