# C++扩展模块的编译与加载

### 前提条件

* GYP项目生成工具——————实现跨平台编译；

* V8引擎C++库。可以实现Javscript与C++的互相调用。

* libuv库，跨平台的一层封装，通过它去调用一些底层操作。（事件循环、文件操作）；

* Node内部库。Node自身提供了一些C++代码，方便面向对象的编程工作。

* 其它库。存在deps目录下的库，在编写扩展模块时可以帮助你，比如zlib,openssl,http_parser等

### C++扩展模块的编写编译

* 无需将源代码编写进Node，而是通过dlopen（）方法动态加载
    - 无需将源代码写进node的命名空间
    - 无需更改src/node_extensions.h文件，在NODE_EXT_LIST_END前添加NODE_EXT_LIST_ITEM(node_hello),从而将node_hello模块添加进node_module_list数组中。
    - 无需更改Node的项目生成文件node.gyp，提供头文件

1. 相关代码编写如下：

```c++
    #include <node.h>
    #include <v8.h>
    using namespace v8;
    // 实现􁇨定义的方法
    Handle<Value> SayHello(const Arguments& args) {
    HandleScope scope;
    return scope.Close(String::New("Hello world!"));
    }
    // 给传入的目􀕔对象􀽁加sayHello()方法
    void Init_Hello(Handle<Object> target) {
    target->Set(String::NewSymbol("sayHello"), FunctionTemplate::New(SayHello)->GetFunction());
    }
    // 调用NODE_MODULE()方法将注􀖩方法定义􀚟内存中
    NODE_MODULE(hello, Init_Hello)
```

2. 配置好***跨平台***GYP相关文件————binding.gyp，相关内容如下：

```c++
  {
    'targets': [
       {
        'target_name': 'hello',
        'sources': [
        'src/hello.cc'
        ],
        'conditions': [
            ['OS == "win"',
                {
                'libraries': ['-lnode.lib']
                }
            ]
        ]
      }
    ]
  }
```

3. 调用 node-gyp configure  ，会在当前目录中创建build目录，生成相关项目文件：

    - linux下，生成Makefile等文件；
    - Windows下，生成vcxproj等文件

4. node-gyp build，输出结果如下：

```c++
    gyp info it worked if it ends with ok
    gyp info using node-gyp@0.8.3
    gyp info using node@0.8.14 | darwin | x64
    gyp info spawn make
    gyp info spawn args [ 'BUILDTYPE=Release', '-C', 'build' ]
    CXX(target) Release/obj.target/hello/hello.o
    SOLINK_MODULE(target) Release/hello.node
    SOLINK_MODULE(target) Release/hello.node: Finished
    gyp info ok
```
### C++扩展模块的加载

 require()方法通过解析标识符、路径分析、文件定位，然后加载执行即可。

1. 保存以下代码为hello.js，调用与运行如下：

``` javascript
    var hello = require('./build/Release/hello.node');
    console.log(hello.sayHello());
```

2. 对于以.node为扩展名的文件，Node将会调用process.dlopen方法去加载文件。而核心模块是采用process.binding('hello')来调用的。会经历以下两个步骤：
   
   （1）uv_dlopen()打开动态链接库
   
   （2）v_dlsym()方法找到动态链接库中通过NODE_MODULE宏定义的方法地址。

上述两个过程都是通过libuv库进行封装的：在linux平台下会调用dlfcn.h头文件中的dlopen()和dlsym（）方法；在Windows平台则是通过LoadLibraryExW()和GetProcAddress（）这个两个方法实现的，它们分别加载.so和.dll文件（实际为.mode文件）

### 总结

* C++扩展模块与Javascript模块的区别在与加载之后不需要编译，直接执行之后就可以被外部调用。

* node-fyp工具的更多细节可以参见https://github.com/TooTallNate/node-gyp（作者为Nathan Rahlich）