# CommonJs的包规范与NPM

### 包结构

```
package.json:包描述文件
bin：可执行二进制文件
lib:javascript代码的目录
doc:用于存放文档的目录
test: 用于存放单元测试用例的代码
```

### 包描述文件与NPM

CommonJS为pachage.json文件定义了一些必需的字段：
```
 name 
 description
 version
 keywords
 maintainers.包维护者列表（name、email和web）

```

NPM实际需要的字段还有author、bin、main、devDependencies

### NPM常用功能

NPM帮助完成了第三方模块的发布、安装和依赖等。

1. 查看帮助
```bash
    npm -v
    npm  //查看所有可添加的选项
    npm <cmd> -h
    bpm  -l
    npm faq commonly asked questions
    npm help <term> search for help on <term>
    npm help npm involved overview
    Specify configs in the ini-formatted file:
    /Users/jacksontian/.npmrc
    or on the command line via: npm <command> --key value
    Config info can be viewed via: npm help config
```

2. 安装依赖包
  
  `npm install express`
  会在当前目录创建node_modules目录，然后在node_modules目录下创建express目录。
  
* 如果需要全局安装，那么需要执行npm install express –g命令进行全局模式安装：
  
  （1） 统一安装到一个目录下

  （2） 通过软连接的方式将bin字段配置的可执行文件链接到Node的可执行目录中。

* 本地安装
```javascript
    npm install <tarball file>
    npm install <tarball url>
    npm install <folder>
```
* 非官方源安装(可通过镜像源安装)

`npm install underscore --registry=http://registry.url`

如果使用过程中几乎都采用镜像源安装，可以执行以下命令指定默认源

`npm config set registry http://registry.url`

* 发布包

(1)编写一个模块：
```javascript
exports.sayHello = function () {
return 'Hello, world.';
};
```
(2) 生成package.json
```javascript
   npm init

    This utility will walk you through creating a package.json file.
    It only covers the most common items, and tries to guess sane defaults.
    See `npm help json` for definitive documentation on these fields
    and exactly what they do.
    Use `npm install <pkg> --save` afterwards to install a package and
    save it as a dependency in the package.json file.
    Press ^C at any time to quit.
    name: (module) hello_test_jackson
    version: (0.0.0) 0.0.1
    description: A hello world package
    entry point: (hello.js) ./hello.js
    test command:
    git repository:
    keywords: Hello world
    author: Jackson Tian
    license: (BSD) MIT
    About to write to /Users/jacksontian/git/diveintonode/examples/03/module/package.json:
    {
    "name": "hello_test_jackson",
    "version": "0.0.1",
    "description": "A hello world package",
    "main": "./hello.js",
    "scripts": {
    "test": "echo \"Error: no test specified\" && exit 1"
    },
    "repository": "",
    "keywords": [
    "Hello",
    "world"
    ],
    "author": "Jackson Tian",
    "license": "MIT"
    }
    Is this ok? (yes) yes
    npm WARN package.json hello_test_jackson@0.0.1 No README.md file found!
```
* 注册包仓库账号
```javascript
$ npm adduser
Username: (jacksontian)
Email: (shyvo1987@gmail.com)
```

* 上传包
```bash
$ npm publish .
npm http PUT http://registry.npmjs.org/hello_test_jackson
npm http 201 http://registry.npmjs.org/hello_test_jackson
npm http GET http://registry.npmjs.org/hello_test_jackson
npm http 200 http://registry.npmjs.org/hello_test_jackson
npm http PUT http://registry.npmjs.org/hello_test_jackson/0.0.1/-tag/latest
npm http 201 http://registry.npmjs.org/hello_test_jackson/0.0.1/-tag/latest
npm http GET http://registry.npmjs.org/hello_test_jackson
npm http 200 http://registry.npmjs.org/hello_test_jackson
npm http PUT
http://registry.npmjs.org/hello_test_jackson/-/hello_test_jackson-0.0.1.tgz/-rev/2-2d64e0946b86687
8bb252f182070c1d5
npm http 201
http://registry.npmjs.org/hello_test_jackson/-/hello_test_jackson-0.0.1.tgz/-rev/2-2d64e0946b86687
8bb252f182070c1d5
+ hello_test_jackson@0.0.1
```

* 安装包

```bash
$ npm install hello_test_jackson --registry=http://registry.npmjs.org
npm http GET http://registry.npmjs.org/hello_test_jackson
npm http 200 http://registry.npmjs.org/hello_test_jackson
hello_test_jackson@0.0.1 ./node_modules/hello_test_jackson
```

* 管理包权限

```bash
$ npm owner ls eventproxy
npm http GET https://registry.npmjs.org/eventproxy
npm http 200 https://registry.npmjs.org/eventproxy
jacksontian <shyvo1987@gmail.com>

npm owner ls <package name>
npm owner add <user> <package name>
npm owner rm <user> <package name>
```

* 分析包

```bash
$ npm ls
/Users/jacksontian
  connect@2.0.3
    crc@0.1.0
        debug@0.6.0
        formidable@1.0.9
         mime@1.2.4
         qs@0.4.2
    hello_test_jackson@0.0.1
    urllib@0.2.3
```