# 第六章 Nodejs理解Buffer

* javascript对字符串操作十分友好，为了在Node中，满足对处理网络协议、接受数据库、处理图片、接受上传文件等操作需求，Buffer对象应运而生。

### Buffer对象特点

* Buffer主要用于操作字节。

* Node在进程启动时就加载了它，并将其放在全局对象上，所以在使用buffer时，无须通过require（）即可直接使用。

* Buffer对象存储数字时的特点：

```javaScript
buf[20] = -100;
console.log(buf[20]); // 156
buf[21] = 300;
console.log(buf[21]); // 44
buf[22] = 3.1415;
console.log(buf[22]); // 3
```

![整数取值](C:\Users\Administrator\Desktop\肖工-嵌入式软件资料-数字政通股份有限公司\nodebook\nodebook\Node.js\asserts\整数取值.png)

### Buffer内存分配

* Node在内存的使用上应用的是在C++层面申请内存，在JavaScript分配内存的策略。

* Node采用slab分配机制,slab是一种动态内存管理机制。slab具有如下3种状态：

```jvascript
full： 完全分配状态
partial： 部分被分配状态
empty： 没有被分配状态
```

* 在JavaScript层面，以slab（大小为8KB的）为单元进行内存的分配。

  （1） Buffer对象都是JavaScript层面的，能够被V8的垃圾回收标记回收，但是其内部的parent属性指向的SlowBuffer对象却来自于Node自身C++中的定义，是C++层面上的Buffer对象，所用内存不在V8的堆中。

  （1）分配小buffer(<8kb)时，采用slab的机制进行预先申请和事后分配，使得JavaScript到操作系统之间不必有太多的内存申请方面的系统调用。

  （2）对于大块的Buffer而言，则直接使用C++层面提供的内存，而无需细腻的分配操作。

  ### Buffer转换

1. 字符串转Buffer

```jvascript
  new Buffer(str, [encoding])//默认是UTF-8编码
或者
  buf.write(string, [offset], [length], [encoding])
```
2. Buffer转字符串
  
```jvascript
  buf.toString([encoding], [start], [end])
  ```

3. 查看Buffer支持的编码类型

  Buffer.isEncoding(encoding)
//发现中国常用的GBK、GB2312和BIG-5编码不支持。

4. 针对Buffer不支持的转码类型，可以使用Node生态圈中的模块完成转换。

（1）iconv 通过C++调用libiconv库完成,采用三级降级策略————尝试翻译无法转换的内容

```jvascript
var iconv = new Iconv('UTF-8', 'ASCII');
iconv.convert('ça va'); // throws EILSEQ
var iconv = new Iconv('UTF-8', 'ASCII//IGNORE');
iconv.convert('ça va'); // returns "a va"
var iconv = new Iconv('UTF-8', 'ASCII//TRANSLIT');
iconv.convert('ça va'); // "ca va"
var iconv = new Iconv('UTF-8', 'ASCII//TRANSLIT//IGNORE');
iconv.convert('ça va 􀈦'); // "ca va "
```

（2）iconv-lite 采用纯JavaScript实现

```jvascript
var iconv = require('iconv-lite');
var str = iconv.decode(buf, 'GB2312');//Buffer转字符串
var buf = iconv.encode("Sample input string", 'win1251');//字符串转Buffer
```
  ### Buffer的拼接
```jvascript
var rs = fs.createReadStream('test.md', { highWaterMark: 11});
rs.setEncoding('utf8');//解决截断，造成的乱码，目前只能处理UTF-8、Base64和
UCS-2/UTF-16LE这3种编码。
解决乱码原因如下：调用setEncoding（）时，可读流内部设置了一个decoder对象，每次通过decoder对象进行Buffer到字符串的解码。
 ```

### 正确拼接Buffer：采用iconv-lite 
```javaScript
var chunks = [];
var size = 0;
res.on('data', function (chunk) {
  chunks.push(chunk);
  size += chunk.length;
  });
res.on('end', function () {
    var buf = Buffer.concat(chunks, size);///关键之处
    var str = iconv.decode(buf, 'utf8');
    console.log(str);
});
```
### Buffer与性能